@@ -48,6 +48,7 @@
 
 #include "archive_platform.h"
 #include "archive_private.h"
+#include "archive_hash.h"
 #include <ctype.h>
 #include <errno.h>
 #include <stddef.h>
@@ -1176,12 +1177,14 @@ la_dosmaperr(unsigned long e)
 	return;
 }
 
-#if !defined(HAVE_OPENSSL_MD5_H) && !defined(HAVE_OPENSSL_SHA_H)
+#if defined(ARCHIVE_HASH_MD5_WIN)    ||\
+    defined(ARCHIVE_HASH_SHA1_WIN)   || defined(ARCHIVE_HASH_SHA256_WIN) ||\
+    defined(ARCHIVE_HASH_SHA384_WIN) || defined(ARCHIVE_HASH_SHA512_WIN)
 /*
  * Message digest functions.
  */
-static void
-Digest_Init(Digest_CTX *ctx, ALG_ID algId)
+void
+__la_hash_Init(Digest_CTX *ctx, ALG_ID algId)
 {
 
 	ctx->valid = 0;
@@ -1202,8 +1205,8 @@ Digest_Init(Digest_CTX *ctx, ALG_ID algId)
 	ctx->valid = 1;
 }
 
-static void
-Digest_Update(Digest_CTX *ctx, const unsigned char *buf, size_t len)
+void
+__la_hash_Update(Digest_CTX *ctx, const unsigned char *buf, size_t len)
 {
 
 	if (!ctx->valid)
@@ -1214,8 +1217,8 @@ Digest_Update(Digest_CTX *ctx, const unsigned char *buf, size_t len)
 		      (DWORD)len, 0);
 }
 
-static void
-Digest_Final(unsigned char *buf, int bufsize, Digest_CTX *ctx)
+void
+__la_hash_Final(unsigned char *buf, size_t bufsize, Digest_CTX *ctx)
 {
 	DWORD siglen = bufsize;
 
@@ -1228,53 +1231,6 @@ Digest_Final(unsigned char *buf, int bufsize, Digest_CTX *ctx)
 	ctx->valid = 0;
 }
 
-#define DIGEST_INIT(name, algid) \
-void name ## _Init(Digest_CTX *ctx)\
-{\
-	Digest_Init(ctx, algid);\
-}
-
-#define DIGEST_UPDATE(name) \
-void name ## _Update(Digest_CTX *ctx, const unsigned char *buf, size_t len)\
-{\
-	Digest_Update(ctx, buf, len);\
-}
-
-#define DIGEST_FINAL(name, size) \
-void name ## _Final(unsigned char *buf, Digest_CTX *ctx)\
-{\
-	Digest_Final(buf, size, ctx);\
-}
-
-DIGEST_INIT(MD5, CALG_MD5)
-DIGEST_UPDATE(MD5)
-DIGEST_FINAL(MD5, MD5_DIGEST_LENGTH)
-
-DIGEST_INIT(SHA1, CALG_SHA1)
-DIGEST_UPDATE(SHA1)
-DIGEST_FINAL(SHA1, SHA1_DIGEST_LENGTH)
-
-/*
- * SHA256 nor SHA384 nor SHA512 are not supported on Windows XP and Windows 2000.
- */
-#ifdef CALG_SHA_256
-DIGEST_INIT(SHA256, CALG_SHA_256)
-DIGEST_UPDATE(SHA256)
-DIGEST_FINAL(SHA256, SHA256_DIGEST_LENGTH)
-#endif
-
-#ifdef CALG_SHA_384
-DIGEST_INIT(SHA384, CALG_SHA_384)
-DIGEST_UPDATE(SHA384)
-DIGEST_FINAL(SHA384, SHA384_DIGEST_LENGTH)
-#endif
-
-#ifdef CALG_SHA_512
-DIGEST_INIT(SHA512, CALG_SHA_512)
-DIGEST_UPDATE(SHA512)
-DIGEST_FINAL(SHA512, SHA384_DIGEST_LENGTH)
-#endif
-
-#endif /* !HAVE_OPENSSL_MD5_H && !HAVE_OPENSSL_SHA_H */
+#endif /* defined(ARCHIVE_HASH_*_WIN) */
 
 #endif /* _WIN32 && !__CYGWIN__ */
